/*
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2010 Igalia S.L.
 * Copyright (c) 2012 NINTENDO.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#include "config.h"
#include "FontCache.h"
#include "NotImplemented.h"
#include <PlatformOS.h>
using namespace LibPlatform;

#include "Font.h"
#include "SimpleFontData.h"
#include "UTF16UChar32Iterator.h"
#include <wtf/Assertions.h>
#include <wtf/text/CString.h>

#define LOG_SIMPLE_RENDER_MSG(vMessage) {}
#define LOG_RENDER_MSG(vMessage, ...) {}

namespace WebCore {

void* createFontConfigPatternForCharacters(const UChar* characters, int length) {
	notImplemented();
    OS::Report("createFontConfigPatternForCharacters");
	return NULL;
}

void* findBestFontGivenFallbacks(const FontPlatformData& fontData, void* pattern)
{
	notImplemented();
    OS::Report("findBestFontGivenFallbacks");
	return 0;
}

SimpleFontData* FontCache::getLastResortFallbackFont(const FontDescription& fontDescription, ShouldRetain shouldRetain)
{
    FontPlatformData* fontPlatformData = 0;
    // We want to return a fallback font here, otherwise the logic preventing FontConfig
    // matches for non-fallback fonts might return 0. See isFallbackFontAllowed.
    const AtomicString serifStr("sans-serif");
    fontPlatformData = getCachedFontPlatformData(fontDescription, serifStr);
    ASSERT(fontPlatformData);
    return getCachedFontData(fontPlatformData, shouldRetain);
}

SimpleFontData* FontCache::getSimilarFontPlatformData(const Font& font)
{
    SimpleFontData* simpleFontData = 0; // does nothing on Wii U

    return simpleFontData;
}

SimpleFontData* SimpleFontData::smallCapsFontData(const FontDescription& fontDescription) const
{
    if (!m_derivedFontData)
        m_derivedFontData = DerivedFontData::create(isCustomFont());
    // FIXME: I think we want to ask FontConfig for the right font again.
    if (!m_derivedFontData->smallCaps)
        m_derivedFontData->smallCaps = createScaledFontData(fontDescription, .7);

    return m_derivedFontData->smallCaps.get();
}

SimpleFontData* SimpleFontData::emphasisMarkFontData(const FontDescription& fontDescription) const
{
    if (!m_derivedFontData)
        m_derivedFontData = DerivedFontData::create(isCustomFont());
    if (!m_derivedFontData->emphasisMark)
        m_derivedFontData->emphasisMark = createScaledFontData(fontDescription, .5);

    return m_derivedFontData->emphasisMark.get();
}

PassOwnPtr<SimpleFontData> SimpleFontData::createScaledFontData(const FontDescription& fontDescription, float scaleFactor) const
{
    return adoptPtr(new SimpleFontData(FontPlatformData(m_platformData, scaleFactor * fontDescription.computedSize()), isCustomFont(), false));
}


void FontCache::getTraitsInFamily(const AtomicString&, Vector<unsigned>&)
{
}

static String getFamilyNameStringFromFontDescriptionAndFamily(const FontDescription& fontDescription, const AtomicString& family)
{
    notImplemented();
    return "";
}

int fontWeightToFontconfigWeight(FontWeight weight)
{
    notImplemented();
    return 0;
}
}
