/*
 * Copyright (C) 2006 Zack Rusin <zack@kde.org>
 * Copyright (C) 2008, 2009 INdT - Instituto Nokia de Tecnologia
 * Copyright (C) 2009-2010 ProFUSION embedded systems
 * Copyright (C) 2009-2010 Samsung Electronics
 * Copyright (c) 2012 NINTENDO.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#include "PlatformMouseEvent.h"
#include <wtf/CurrentTime.h>

namespace WebCore {

void PlatformMouseEvent::setClickCount(unsigned int count)
{
    m_clickCount = count;
}

PlatformMouseEvent::PlatformMouseEvent(WiiUEventType_t type, const WiiUMouseEvent * ev)
{
    PlatformMouseEvent();

    m_position.setX(ev->x);
    m_position.setY(ev->y);
    
    m_globalPosition = m_position;

    switch(type) {
        case WIIU_MOUSE_DOWN:
            m_eventType = MouseEventPressed;
            break;

        case WIIU_MOUSE_UP:
            m_eventType = MouseEventReleased;
            break;

        case WIIU_MOUSE_MOVE:
            m_eventType = MouseEventMoved;
            break;

        case WIIU_TOUCH_MOVE:
        	m_eventType = TouchEventMoved;
        	break;

        case WIIU_TOUCH_DOWN:
        	m_eventType = TouchEventPressed;
        	break;

        case WIIU_TOUCH_UP:
        	m_eventType = TouchEventReleased;
        	break;

        default:
            break;

    }
    m_timestamp = ev->timestamp;
    if(ev->button == 0)
    	m_button = LeftButton;
    else
    	m_button = NoButton;
	m_clickCount = ev->clickCount;

	buttonState = ev->state;
}

}
