/*
 * This file is part of the WebKit project.
 *
 * Copyright (C) 2006 Apple Computer, Inc.
 * Copyright (C) 2006 Michael Emmel mike.emmel@gmail.com
 * Copyright (C) 2007 Holger Hans Peter Freyther
 * Copyright (C) 2007 Alp Toker <alp@atoker.com>
 * Copyright (c) 2012 NINTENDO.
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#ifndef RenderThemeWiiU_h
#define RenderThemeWiiU_h

#include "Color.h"
#include "IntSize.h"
#include "RenderTheme.h"

#if !PLATFORM(WIIU)
#include <string>
#endif

namespace WebCore {

class RenderThemeWiiU : public RenderTheme {
public:
    RenderThemeWiiU();

    #if !PLATFORM(WIIU)
    // Set the base path for loading theme assets
    static void setAssetsPath(const char *set);
    static std::string prependAssetsPath(const char *path);
    #endif

    // A method asking if the theme is able to draw the focus ring.
    virtual bool supportsFocusRing(const RenderStyle*) const;

    virtual Color platformActiveSelectionBackgroundColor() const;

    // A method to obtain the baseline position for a "leaf" control.  This will only be used if a baseline
    // position cannot be determined by examining child content. Checkboxes and radio buttons are examples of
    // controls that need to do this.
    virtual int baselinePosition(const RenderObject*) const;

#if ENABLE(VIDEO)
    virtual String extraMediaControlsStyleSheet();
#endif

    // System fonts.
    virtual void systemFont(int propId, FontDescription&) const;
    

    virtual int popupInternalPaddingLeft(RenderStyle*) const;
    virtual int popupInternalPaddingRight(RenderStyle*) const;
    virtual int popupInternalPaddingTop(RenderStyle*) const;
    virtual int popupInternalPaddingBottom(RenderStyle*) const;
    
    virtual String extraDefaultStyleSheet();

protected:
    virtual bool paintCheckbox(RenderObject* o, const PaintInfo& i, const IntRect& r);
    virtual void setCheckboxSize(RenderStyle* style) const;

    virtual bool paintRadio(RenderObject* o, const PaintInfo& i, const IntRect& r);
    virtual void setRadioSize(RenderStyle* style) const;

    virtual void setButtonSize(RenderStyle* style) const;
    virtual void adjustButtonStyle(CSSStyleSelector*, RenderStyle*, Element*) const;
    virtual bool paintButton(RenderObject*, const PaintInfo&, const IntRect&);

    virtual void adjustInnerSpinButtonStyle(CSSStyleSelector* selector, RenderStyle* style, Element* e) const;
    virtual bool paintInnerSpinButton(RenderObject* o, const PaintInfo& i, const IntRect& rect);

    virtual void adjustTextFieldStyle(CSSStyleSelector*, RenderStyle*, Element*) const;
    virtual bool paintTextField(RenderObject*, const PaintInfo&, const IntRect&);

    virtual void adjustTextAreaStyle(CSSStyleSelector*, RenderStyle*, Element*) const;
    virtual bool paintTextArea(RenderObject*, const PaintInfo&, const IntRect&);

    virtual void adjustMenuListStyle(CSSStyleSelector*, RenderStyle*, Element*) const;
    virtual bool paintMenuList(RenderObject*, const PaintInfo&, const IntRect&);
    

    virtual bool paintMenuListButton(RenderObject*, const PaintInfo&, const IntRect&);

	virtual void adjustSliderThumbSize(RenderStyle*) const;

    virtual bool paintSliderTrack(RenderObject*, const PaintInfo&, const IntRect&);

    virtual bool paintSliderThumb(RenderObject*, const PaintInfo&, const IntRect&);

    virtual void adjustSearchFieldResultsDecorationStyle(CSSStyleSelector*, RenderStyle*, Element*) const;
    virtual bool paintSearchFieldResultsDecoration(RenderObject*, const PaintInfo&, const IntRect&);

    virtual void adjustSearchFieldStyle(CSSStyleSelector*, RenderStyle*, Element*) const;
    virtual bool paintSearchField(RenderObject*, const PaintInfo&, const IntRect&);

    virtual void adjustSearchFieldResultsButtonStyle(CSSStyleSelector*, RenderStyle*, Element*) const;
    virtual bool paintSearchFieldResultsButton(RenderObject*, const PaintInfo&, const IntRect&);

    virtual void adjustSearchFieldCancelButtonStyle(CSSStyleSelector*, RenderStyle*, Element*) const;
    virtual bool paintSearchFieldCancelButton(RenderObject*, const PaintInfo&, const IntRect&);

	virtual double caretBlinkInterval() const { return 1.0; }

#if ENABLE(PROGRESS_TAG)
    virtual double animationRepeatIntervalForProgressBar(RenderProgress*) const;
    virtual double animationDurationForProgressBar(RenderProgress*) const;
    virtual void adjustProgressBarStyle(CSSStyleSelector*, RenderStyle*, Element*) const;
    virtual bool paintProgressBar(RenderObject*, const PaintInfo&, const IntRect&);
    static IntRect calculateProgressRect(RenderObject*, const IntRect&);
#endif

#if ENABLE(VIDEO)
    virtual bool paintMediaButton(int, RenderObject*, const PaintInfo&, const IntRect&, int);
    virtual bool paintMediaFullscreenButton(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaPlayButton(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaOverlayPlayButton(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaMuteButton(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaSeekBackButton(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaSeekForwardButton(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaSliderTrack(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaSliderThumb(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaVolumeSliderContainer(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaVolumeSliderTrack(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaVolumeSliderThumb(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaRewindButton(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaReturnToRealtimeButton(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaToggleClosedCaptionsButton(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaControlsBackground(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaCurrentTime(RenderObject*, const PaintInfo&, const IntRect&);
    virtual bool paintMediaTimeRemaining(RenderObject*, const PaintInfo&, const IntRect&);
#endif

public:

/*
    This will load an image from disk.  It is based upon the code for Image::loadPlatformResource 
    but without a general policy about where to locate content, we instead have a function which
    takes a full path.  

    Also, we may eventually embed the images directly in the executable to keep them from being
    easily modified, in which case this method can change to load then from an executable.
*/
    static PassRefPtr<Image> loadRenderThemeImage(const char *path);
    static PassRefPtr<Image> loadRenderThemeImageFromMemory(const unsigned char *image, int size);
/*
    These two methods will draw the image in such away that either the vertical or horizontal edges
    are not interpolated to keep them "crisp" even if the image itself is scaled.
*/ 
    static void paintImagePreserveHorizontalEdges(GraphicsContext* const ctx, Image *image, FloatRect &frect);

    static void paintImagePreserveVerticalEdges(GraphicsContext* const ctx, Image *image, FloatRect &frect);

private:

	const Color defaultBorderColor;
	const Color defaultButtonBGColor;
	
	const IntSize defaultButtonCorner;

    /* CRANK: DLM resouces for controls */
    RefPtr<Image> imgButtonPressed;
    RefPtr<Image> imgButtonNormal;
    RefPtr<Image> imgButtonDisabled;

    RefPtr<Image> imgMenuListButtonPressed;
    RefPtr<Image> imgMenuListButtonNormal;
    RefPtr<Image> imgMenuListButtonDisabled;

    RefPtr<Image> imgCheckBoxChecked;
    RefPtr<Image> imgCheckBoxCheckedDisabled;
    RefPtr<Image> imgCheckBoxUnchecked;
    RefPtr<Image> imgCheckBoxUncheckedDisabled;

    RefPtr<Image> imgRadioButtonChecked;
    RefPtr<Image> imgRadioButtonCheckedDisabled;
    RefPtr<Image> imgRadioButtonUnchecked;
    RefPtr<Image> imgRadioButtonUncheckedDisabled;

    RefPtr<Image> imgPlayButton;
    RefPtr<Image> imgPauseButton;
    RefPtr<Image> imgMuteButton;

    RefPtr<Image> imgSpinnerUpButton;
    RefPtr<Image> imgSpinnerUpButtonPressed;
    RefPtr<Image> imgSpinnerUpButtonDisabled;

    RefPtr<Image> imgSpinnerDownButton;
    RefPtr<Image> imgSpinnerDownButtonPressed;
    RefPtr<Image> imgSpinnerDownButtonDisabled;



	
	IntRect       m_mediaSliderRect; 
	double		  m_mediaTimelineOverlayAlpha;
};

}

#endif
