/*
 * Copyright (c) 2013 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WiiUMediaPlugin_h
#define WiiUMediaPlugin_h

#include <config.h>

namespace WebCore {

// -------------------------------------------------------------------------------------
// Communication interface between WebCore/WiiU Media Player and WiiU Launcher
//
// Info: Please note that this is compiled into the "libwebkitwiiu_rpl".
// 	     The symbols which the Launcher needs to access need to be exported via "wiiu-rpl-build\libwebkitwiiu.def"
class WiiUMediaInterfacePluginSystem
	{
public:
	//! Collection of events coming from the Media Player
	class Listener
		{
	public:
		//! Start the iteration for mime-types
		virtual void BeginAskingForMimeType(void) = 0;

		//! Ask for one mime-type. The client returns NULL if the iteration ends
		virtual const char* AskForNextMimeType(void) = 0;

		//! Finished with iteration of mime-types
		virtual void EndAskingForMimeType(void) = 0;
		};
		
	//! Set our "event receicer"
	static void SetListener(Listener* pListener);		

	//! Get current "event receiver"
	static Listener* GetListener(void);		

private:
	static Listener* mpListener;
	};

};

#endif

