/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "wk_event.h"
#include "WebViewWiiU.h"

// ----------------------------------------------------------------------------------

bool WiiUTimerFired::s_bInQueue;
bool WiiUInputUpdate::s_bInQueue;
bool WiiUDisplayVsyncEvent::s_bInQueue[WIIU_MAX_DISPLAY];

bool WiiUMouseData::s_bMouseMoveInQueue[WIIU_MAX_DISPLAY];
WiiUMouseData* WiiUMouseData::s_LastMouseMoveEvent[WIIU_MAX_DISPLAY];

bool WiiUTouchData::s_bTouchMoveInQueue[WIIU_MAX_DISPLAY];
WiiUTouchData* WiiUTouchData::s_LastTouchMoveEvent[WIIU_MAX_DISPLAY];

// ----------------------------------------------------------------------------------

void ResetWkEventGlobals()
{
	unsigned int i;

	WiiUTimerFired::s_bInQueue = false;
	WiiUInputUpdate::s_bInQueue = false;

	for (i = 0; i < WIIU_MAX_DISPLAY; i++)
	{
		WiiUDisplayVsyncEvent::s_bInQueue[i] = 0;
		WiiUMouseData::s_bMouseMoveInQueue[i] = 0;
		WiiUMouseData::s_LastMouseMoveEvent[i] = 0;
		WiiUTouchData::s_bTouchMoveInQueue[i] = 0;
		WiiUTouchData::s_LastTouchMoveEvent[i] = 0;
	}
}

// ----------------------------------------------------------------------------------
/**
*/
void wk_InitEvents()
{
	WiiUTimerFired::Startup();
	WiiUInputUpdate::Startup();
	WiiUDisplayVsyncEvent::Startup();
	WiiUMouseData::Startup();
	WiiUTouchData::Startup();
}

// ----------------------------------------------------------------------------------
/**
*/
void wk_ExitEvents()
{
	// nothing to for now
}

// ----------------------------------------------------------------------------------
/**
*/
void WiiUTimerFired::Startup()
{
	s_bInQueue = false;
}

// ----------------------------------------------------------------------------------
/**
*/
bool WiiUTimerFired::rejectExclusive() const
{
	return s_bInQueue;
}

// ----------------------------------------------------------------------------------
/**
*/
void WiiUTimerFired::postExclusive()
{
	s_bInQueue = true;
}

// ----------------------------------------------------------------------------------
/**
*/
void WiiUTimerFired::releaseExclusive()
{
	s_bInQueue = false;
}


void WiiUInputUpdate::Startup()
{
    s_bInQueue = false;
}

// ----------------------------------------------------------------------------------
/**
*/
bool WiiUInputUpdate::rejectExclusive() const
{
    return s_bInQueue;
}

// ----------------------------------------------------------------------------------
/**
*/
void WiiUInputUpdate::postExclusive()
{
    s_bInQueue = true;
}

// ----------------------------------------------------------------------------------
/**
*/
void WiiUInputUpdate::releaseExclusive()
{
    s_bInQueue = false;
}
// ----------------------------------------------------------------------------------
/**
*/
void WiiUDisplayVsyncEvent::Startup()
{
	for(int i=0; i<WIIU_MAX_DISPLAY; ++i)
		s_bInQueue[i] = false;
}

// ----------------------------------------------------------------------------------
/**
*/
bool WiiUDisplayVsyncEvent::rejectExclusive() const
{
	WebView *pView = (WebView*)webview;
	int idx = pView->GetRenderToTV() ? 0 : (1 + pView->GetGamePadIndex());
	ASSERT(idx < WIIU_MAX_DISPLAY);
	return s_bInQueue[idx];
}

// ----------------------------------------------------------------------------------
/**
*/
void WiiUDisplayVsyncEvent::postExclusive()
{
	WebView *pView = (WebView*)webview;
	int idx = pView->GetRenderToTV() ? 0 : (1 + pView->GetGamePadIndex());
	ASSERT(idx < WIIU_MAX_DISPLAY);
	s_bInQueue[idx] = true;
}

// ----------------------------------------------------------------------------------
/**
*/
void WiiUDisplayVsyncEvent::releaseExclusive()
{
	WebView *pView = (WebView*)webview;
	int idx = pView->GetRenderToTV() ? 0 : (1 + pView->GetGamePadIndex());
	ASSERT(idx < WIIU_MAX_DISPLAY);
	s_bInQueue[idx] = false;
}

// ----------------------------------------------------------------------------------
/**
*/
void WiiUMouseData::Startup()
{
    for(int i=0; i<WIIU_MAX_DISPLAY; ++i) {
        s_LastMouseMoveEvent[i] = NULL;
        s_bMouseMoveInQueue[i] = false;
    }
}

// ----------------------------------------------------------------------------------
/**
*/
bool WiiUMouseData::rejectExclusive() const
{

    if (evType != WIIU_MOUSE_MOVE)
        return false;

    WebView *pView = (WebView*)webview;
    int idx = pView->GetRenderToTV() ? 0 : (1 + pView->GetGamePadIndex());
    ASSERT(idx < WIIU_MAX_DISPLAY);

    //Only want exclusiveness on the mouse move.
    if (!s_bMouseMoveInQueue[idx] || !s_LastMouseMoveEvent[idx] || (ev.state != s_LastMouseMoveEvent[idx]->ev.state)) {
        return false;
    }

    //Update Mouse Move Data
    s_LastMouseMoveEvent[idx]->ev.x = ev.x;
    s_LastMouseMoveEvent[idx]->ev.y = ev.y;
    s_LastMouseMoveEvent[idx]->ev.button = ev.button;
    s_LastMouseMoveEvent[idx]->ev.state = ev.state;
    s_LastMouseMoveEvent[idx]->ev.clickCount = ev.clickCount;

    return true;
}

// ----------------------------------------------------------------------------------
/**
*/
void WiiUMouseData::postExclusive()
{
    if (evType != WIIU_MOUSE_MOVE)
        return;

    WebView *pView = (WebView*)webview;
    int idx = pView->GetRenderToTV() ? 0 : (1 + pView->GetGamePadIndex());
    ASSERT(idx < WIIU_MAX_DISPLAY);

    s_LastMouseMoveEvent[idx] = this;
    s_bMouseMoveInQueue[idx] = true;
}

// ----------------------------------------------------------------------------------
/**
*/
void WiiUMouseData::releaseExclusive()
{
   if (evType != WIIU_MOUSE_MOVE)
        return;

    WebView *pView = (WebView*)webview;
    int idx = pView->GetRenderToTV() ? 0 : (1 + pView->GetGamePadIndex());
    ASSERT(idx < WIIU_MAX_DISPLAY);

    if (s_LastMouseMoveEvent[idx] == this) {
        s_bMouseMoveInQueue[idx] = false;
        s_LastMouseMoveEvent[idx] = NULL;
    }
}

void WiiUTouchData::Startup()
{
    for(int i=0; i<WIIU_MAX_DISPLAY; ++i) {
        s_LastTouchMoveEvent[i] = NULL;
        s_bTouchMoveInQueue[i] = false;
    }
}

// ----------------------------------------------------------------------------------
/**
*/
bool WiiUTouchData::rejectExclusive() const
{

    if (evType != WIIU_TOUCH_MOVE)
        return false;

    WebView *pView = (WebView*)webview;
    int idx = pView->GetRenderToTV() ? 0 : (1 + pView->GetGamePadIndex());
    ASSERT(idx < WIIU_MAX_DISPLAY);

    //Only want exclusiveness on the mouse move.
    if (!s_bTouchMoveInQueue[idx] || !s_LastTouchMoveEvent[idx] || (ev.state != s_LastTouchMoveEvent[idx]->ev.state)) {
        return false;
    }

    //Update Mouse Move Data
    s_LastTouchMoveEvent[idx]->ev.x = ev.x;
    s_LastTouchMoveEvent[idx]->ev.y = ev.y;
    s_LastTouchMoveEvent[idx]->ev.button = ev.button;
    s_LastTouchMoveEvent[idx]->ev.state = ev.state;
    s_LastTouchMoveEvent[idx]->ev.clickCount = ev.clickCount;

    return true;
}

// ----------------------------------------------------------------------------------
/**
*/
void WiiUTouchData::postExclusive()
{
    if (evType != WIIU_TOUCH_MOVE)
        return;

    WebView *pView = (WebView*)webview;
    int idx = pView->GetRenderToTV() ? 0 : (1 + pView->GetGamePadIndex());
    ASSERT(idx < WIIU_MAX_DISPLAY);

    s_LastTouchMoveEvent[idx] = this;
    s_bTouchMoveInQueue[idx] = true;
}

// ----------------------------------------------------------------------------------
/**
*/
void WiiUTouchData::releaseExclusive()
{
   if (evType != WIIU_TOUCH_MOVE)
        return;

    WebView *pView = (WebView*)webview;
    int idx = pView->GetRenderToTV() ? 0 : (1 + pView->GetGamePadIndex());
    ASSERT(idx < WIIU_MAX_DISPLAY);

    if (s_LastTouchMoveEvent[idx] == this) {
        s_bTouchMoveInQueue[idx] = false;
        s_LastTouchMoveEvent[idx] = NULL;
    }
}
