/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WIIUAUTOSCROLLER_H_
#define WIIUAUTOSCROLLER_H_

#include <Timer.h>

namespace WebCore {
	class Node;
	class Frame;
	class FrameView;
	class RenderLayer;
}

class WiiUAutoScroller {
public:
	WiiUAutoScroller();
	~WiiUAutoScroller();

	void setAcceleration(float acceleration) {m_acceleration = acceleration;}

	void startScrollWithVelocity(float x_velocity, float y_velocity);
	void stopScrollWithVelocity();

    void smoothScroll(float dx, float dy, float vel, float acc, float diffuse, float screen_x, float screen_y);
    void dpadScroll(float vec_x, float vec_y, float screen_x, float screen_y);
    
	bool isScrolling() const {return m_isScrolling;}
	bool isScrollingForFocusNav() const {return m_isScrollingForNav;}
	void setIsScrollingForFocusNav(bool isScrollingForFocusNav) { m_isScrollingForNav = isScrollingForFocusNav; }


	bool shouldScrollWithVelocity(WebCore::Frame *mainFrame, int dx, int dy);

	void setScrollNode(WebCore::Node *node);
	void scrollNode(int dx, int dy);

	WebCore::Node* getScrollNode() { return m_scrollNode; }
private:

	static const int defaultFrameRateHz = 60;
	

	void scrollEnded();
	void scrollProgressed();
	void scrollCanceled();
	void scrollVelocity(WebCore::Timer<WiiUAutoScroller>*);
	
	void scrollMainFrame(int dx, int dy);

	void setScrollNodeToMainDocument();

	float calculateTween(float state);
	float calculateInterval(int frameRate);

	float getDelta(float v0, float a, float t1, float t2);
	float getPosition(float v0, float a, float t);

	bool canScrollLayer(WebCore::RenderLayer *layer, int scrollX, int scrollY);
	bool canScrollView(WebCore::FrameView *view, int dx, int dy);

	static const double scrollTimeIncrement;
    void smoothScrollUpdate(WebCore::Timer<WiiUAutoScroller>*);
    void dpadScrollUpdate(WebCore::Timer<WiiUAutoScroller>*);

	WebCore::Node* m_scrollNode;

	double m_remainingX;
	double m_remainingY;

	float m_XVelocity;
	float m_YVelocity;
	float m_lastTime;
	float m_duration;
	float m_acceleration;

	bool  m_isScrolling;
	bool  m_isScrollingForNav;
	float m_currentStepValue;
	float m_startStepValue;
	float m_endStepValue;
	float m_animationDuration;
	float m_animationInterval;
	float m_elapsedTime;
	WebCore::Timer<WiiUAutoScroller> m_velocityTimer;
	WebCore::Timer<WiiUAutoScroller> m_smoothScrollTimer;
    WebCore::Timer<WiiUAutoScroller> m_dpadScrollTimer;

    // stick scroll vars
    double time_start;
    float rel_x;
    float rel_y;
    float vec_x;
    float vec_y;
    float vel;
    float acc;
    float diffuse;
    static float slowing;
    float screen_x;
    float screen_y;
    float vel_x;
    float vel_y;
    float cos_theta;

    bool faster;

public:
    bool dragging;
};


#endif /* WIIUAUTOSCROLLER_H_ */
