#ifndef CONFIG_H
#define CONFIG_H

/**
	This namespace is for holding onto settings from config.xml
*/
#include <string>
#include <vector>
#include <PlatformDefines.h>

namespace LibPlatform 
{
    struct WindowRedrawCallback {
    WindowRedrawCallback(void) : func(0), args(0) {}
        void (*func)(void *);
        void *args;
    };
    
    struct DisplayRemapInfo
    {
        int srcX;
        int srcY;
        int srcWidth;
        int srcHeight;
        int dstX;
        int dstY;
        int dstWidth;
        int dstHeight;
    };

    enum ViewportFilterMode
    {
        POINT_FILTER,
        BILINEAR_FILTER
    };

	enum DisplayType
	{
		DISPLAY_TYPE_TV,
		DISPLAY_TYPE_GAMEPAD0,
		DISPLAY_TYPE_ALL,
		DISPLAY_TYPE_NONE,
		NUM_DISPLAY_TYPES
	};
    enum SelectElementMode
    {
        MODE_TYPE_NON_MODAL,
        MODE_TYPE_MODAL
    };


    enum HoldSwipeGestureType 
	{
        HOLD_SWIPE_TO_SCROLL,
        HOLD_SWIPE_TO_SELECT,
    }; 
       
	namespace ApplicationConfig 
	{
		struct FileHook	// pretty much the same thing in the mediaTransportFileHook
		{
		public:
			virtual long long GetFileSize(int fd) = 0;
			virtual int Open(const char *url) = 0;
			virtual int Read(int fd, long long offset, void *buffer, int count) = 0;
			virtual void Close(int fd) = 0;
		};
		void SetMediaPlayer(int mediaPlayer);
		int GetMediaPlayer();
		void SetOverrideURLs(const char *tv, const char *gp, const char * shared);
		void GetOverrideURLs(const char **tv, const char **gp, const char **shared);
        void SetIsLandscapeOrientation(bool isLandscape);
        bool GetIsLandscapeOrientation();
        void SetIsPrimaryOrientation(bool isPrimary);
        bool GetIsPrimaryOrientation();
        void SetGPDisplayDimensions(int GPDisplayWidth, int GPDisplayHeight);
        void GetGPDisplayDimensions(int &outGPDisplayWidth, int &outGPDisplayHeight);
        void SetTVDisplayDimensions(int TVDisplayWidth, int TVDisplayHeight);
        void GetTVDisplayDimensions(int &outTVDisplayWidth, int &outTVDisplayHeight);
        void SetExtendedDOMSize(int DOMWidth, int DOMHeight);
        void GetExtendedDOMSize(int &outDOMWidth, int &outDOMHeight);
        void SetExtendedDisplay(bool extendedDisplay);
        bool GetExtendedDisplay();
		void SetIsServiceApplication(bool serviceApp);
        bool GetIsServiceApplication();
		void SetNEXGameServerID(int serverID);
		int GetNEXGameServerID();
		void SetClearColor(unsigned r, unsigned g, unsigned b);
		bool GetClearColor(int &rOut, int &gOut, int &bOut);
		void SetNEXAccessKey(const wchar_t * accessKey);
		const wchar_t* GetNEXAccessKey();
        void SetTVRemapInfo(int srcX, int srcY, int srcWidth, int srcHeight, int dstX, int dstY, int dstWidth, int dstHeight);
        void SetGP0RemapInfo(int srcX, int srcY, int srcWidth, int srcHeight, int dstX, int dstY, int dstWidth, int dstHeight);
        const DisplayRemapInfo* GetTVRemapInfo();
        const DisplayRemapInfo* GetGP0RemapInfo();
        bool IsTVRemapDirty();
        bool IsGP0RemapDirty();
        void SetTVViewportFilterMode(ViewportFilterMode filterMode);
        void SetGP0ViewportFilterMode(ViewportFilterMode filterMode);
        ViewportFilterMode GetTVViewportFilterMode();
        ViewportFilterMode GetGP0ViewportFilterMode();
		DisplayType GetNavigationEnabledTarget(LibPlatform::ControllerID id);
		void SetNavigationEnabledTarget(LibPlatform::ControllerID id, DisplayType target);
		DisplayType GetFocusNavigationEnabledTarget(LibPlatform::ControllerID id);
		void SetFocusNavigationEnabledTarget(LibPlatform::ControllerID id, DisplayType target);
		DisplayType GetScrollEnabledTarget(LibPlatform::ControllerID id);
		void SetScrollEnabledTarget(LibPlatform::ControllerID id, DisplayType target);
		DisplayType GetVideoControlEnabledTarget(LibPlatform::ControllerID id);
		void SetVideoControlEnabledTarget(LibPlatform::ControllerID id, DisplayType target);
        bool GetVideoControlsEnabled();
        void SetVideoControlsEnabled(bool isEnabled);
		void GetFocusRingColor(int &rOut, int &gOut, int &bOut);
        
        void SetHoldSwipeTouchGestureBehaviour(HoldSwipeGestureType type); 
        HoldSwipeGestureType  GetHoldSwipeTouchGestureBehaviour(); 
        void SetMouseMoveDelay(int timeMs);
        int GetMouseMoveDelay(); 
        long GetMouseMoveDelayTicks(); 
        void SetMouseDownDelay(int timeMs);
        int GetMouseDownDelay();  
        long GetMouseDownDelayTicks();         
        void SetDoubleClickDelay(int timeMs); 
        int GetDoubleClickDelay(); 
        long GetDoubleClickDelayTicks(); 
        void SetDoubleClickRadius(int pixels); 
        int GetDoubleClickRadius(); 
        void SetScrollDetectionRadius(int pixels); 
        int GetScrollDetectionRadius(); 
		int GetScrollDetectionRadiusScaledForTV();

		double GetTVVolume();
		void SetTVVolume(double volumeLevel);
		double GetGPVolume();
		void SetGPVolume(double volumeLevel);

        DisplayType GetSelectElementEnabledTarget();
		void SetSelectElementEnabledTarget(DisplayType target);
        SelectElementMode GetSelectElementMode(); 
        void SetSelectElementMode(SelectElementMode mode);
		DisplayType GetDialogPromptEnabledTarget();
		void SetDialogPromptEnabledTarget(DisplayType target);
        bool GetAuthDialogPromptEnabled();
        void SetAuthDialogPromptEnabled(bool isEnabled);
        bool GetConfirmDialogPromptEnabled();
        void SetConfirmDialogPromptEnabled(bool isEnabled);
        bool GetAlertDialogPromptEnabled();
        void SetAlertDialogPromptEnabled(bool isEnabled);
        bool GetLastViewNavigatedIsTV();
        void SetLastViewNavigatedIsTV(bool isTV);
        bool GetEscapeIsPressed();
        void SetEscapeIsPressed(bool isPressed);
        
        void SetAntiAliasing(int setting);
        int GetAntiAliasing();

        void SetTouchScrollScrollEnabled(DisplayType target, bool isEnabled);
        bool GetTouchScrollScrollEnabled(DisplayType target); 
        void SetTouchScrollScrollBarsEnabled(DisplayType target, bool isEnabled);
        bool GetTouchScrollScrollBarsEnabled(DisplayType target);
        void SetTouchScrollTouchEmulated(DisplayType target, bool isEnabled);
        bool GetTouchScrollTouchEmulated(DisplayType target);
        
        void RegisterFileHook(FileHook *hookP);
		FileHook* GetFileHook(void);
        void RegisterControllerRemapFilePath(const std::string &JSONInputRemapFilePath);
        std::vector<const std::string>* GetControllerRemapFilePathList();
        void ClearAllControllerRemapFilePaths();
        bool SetControllerRemap(const std::string *inputDevice, const std::string *deviceButton, const std::string *keyboardKey);
        signed GetControllerRemap(ControllerID ID, unsigned deviceButton);
        void ClearControllerRemaps(ControllerID ID);
		void ClearAllControllerRemaps(void);
		void ClearRemappedButtons(ControllerID ID, unsigned *pDeviceButtons);

        void SetFFLSM(bool);
        bool GetFFLSM(void);

        void SetMemoryCheckInfo(unsigned frames_per_check, unsigned mem_bytes_needed_or_panic, unsigned mem_bytes_needed_or_fatal, unsigned code_bytes_needed_or_fatal);
        void GetMemoryCheckInfo(unsigned &frames_per_check, unsigned &mem_bytes_needed_or_panic, unsigned &mem_bytes_needed_or_fatal,unsigned &code_bytes_needed_or_fatal);

	
		//PlayReport
		void SetPlayReportTaskId(const std::string &taskId);
		void SetPlayReportNumKeys(const int numKeys);
		std::string *GetPlayReportTaskId();
		int GetPlayReportNumKeys();

        void WindowSetTVCallback(void (*func)(void *), void *args);
        void WindowSetGPCallback(void (*func)(void *), void *args);
        void WindowStateReset(void);
        void WindowLoadingGP(void);
        void WindowLoadingTV(void);
        void WindowDoneGP(void);
        void WindowDoneTV(void);
        bool WindowsDone(void);
    }
}

#endif//CONFIG_H
